unit UGFX;

interface
uses DGLOpenGL;

type
   TArray3f = array[0..2] of TGLfloat;
   TArray4f = array[0..3] of TGLfloat;

   TGrid = class
     public
       Active         : Boolean;
       inActiveColor  : TArray3f;
       ActiveColor    : TArray3f;
       BGColor_Light  : TArray4f;
       BGColor_Dark   : TArray4f;
       l,b            : integer;

       constructor Create;
       procedure drawGrid;
       procedure drawArrows;
   end;

   procedure drawScene(Step : integer);

implementation
uses UMain;

constructor TGrid.Create;
begin
     inherited;
     active           := true;

     l                := 10;
     b                := 10;

     inActiveColor[0] := 0.5;
     inActiveColor[1] := 0.5;
     inActiveColor[2] := 0.5;

     ActiveColor[0]   := 0.9;
     ActiveColor[1]   := 0.0;
     ActiveColor[2]   := 0.7;

     BGColor_Light[0] := 0.7;
     BGColor_Light[1] := 0.2;
     BGColor_Light[2] := 0.6;
     BGColor_Light[3] := 0.75;

     BGColor_Dark[0]  := 0.2;
     BGColor_Dark[1]  := 0.0;
     BGColor_Dark[2]  := 0.1;
     BGColor_Dark[3]  := 0.75;
end;

procedure TGrid.drawGrid;
var i,j                      : integer;
    intens1, intens2         : real;
    color                    : TArray3f;
begin
     if active then
       begin
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glBegin(GL_QUADS);
              glColor4fv(@BGColor_Light);
              glVertex3f(-l div 2, 0,  b div 2);
              glVertex3f( l div 2, 0,  b div 2);
              glColor4fv(@BGColor_Dark);
              glVertex3f( l div 2, 0, -b div 2);
              glVertex3f(-l div 2, 0, -b div 2);
            glEnd;
            glDisable(GL_BLEND);

            color := ActiveColor;
       end else
       begin
            color := InActiveColor;
       end;

     glDisable(GL_DEPTH_TEST);
     for i := (-l div 2) to (l div 2)-1 do
       for j := (-b div 2) to (b div 2)-1 do
         begin
              intens2 := ( (i+ (l div 2)) + (j+ (b div 2)) );
              if intens2 = 0 then intens2 := 1/(l+b);
              intens1 := 2*intens2 / (l+b);
              intens2 := intens1 + ( 1/(l+b) );

              glBegin(GL_LINES);
                glColor3f(color[0]*intens1, color[1]*intens1, color[2]*intens1);
                glVertex3f(i,0,j);

                glColor3f(color[0]*intens2, color[1]*intens2, color[2]*intens2);
                glVertex3f(i+1,0,j);

                glColor3f(color[0]*intens1, color[1]*intens1, color[2]*intens1);
                glVertex3f(i,0,j);

                glColor3f(color[0]*intens2, color[1]*intens2, color[2]*intens2);
                glVertex3f(i,0,j+1);
              glEnd;
         end;

     glLineWidth(2);
     for i := (-l div 2) to (l div 2)-1 do
         begin
              intens2 := (i+ (l div 2)) + b;
              if intens2 = 0 then intens2 := 1/(l+b);
              intens1 := 2*intens2 / (l+b);
              intens2 := intens1 + ( 1/(l+b) );

              glBegin(GL_LINES);
                glColor3f(color[0]*intens1, color[1]*intens1, color[2]*intens1);
                glVertex3f(i,0,b div 2);

                glColor3f(color[0]*intens2, color[1]*intens2, color[2]*intens2);
                glVertex3f(i+1,0,b div 2);
              glEnd;

              intens2 := (i+ (l div 2));
              if intens2 = 0 then intens2 := 1/(l+b);
              intens1 := 2*intens2 / (l+b);
              intens2 := intens1 + ( 1/(l+b) );

              glBegin(GL_LINES);
                glColor3f(color[0]*intens1, color[1]*intens1, color[2]*intens1);
                glVertex3f(i,0,-b div 2);

                glColor3f(color[0]*intens2, color[1]*intens2, color[2]*intens2);
                glVertex3f(i+1,0,-b div 2);
              glEnd;
         end;
     for j := (-b div 2) to (b div 2)-1 do
         begin
              intens2 := (j+ (b div 2)) + l;
              if intens2 = 0 then intens2 := 1/(l+b);
              intens1 := 2*intens2 / (l+b);
              intens2 := intens1 + ( 1/(l+b) );

              glBegin(GL_LINES);
                glColor3f(color[0]*intens1, color[1]*intens1, color[2]*intens1);
                glVertex3f(l div 2,0,j);

                glColor3f(color[0]*intens2, color[1]*intens2, color[2]*intens2);
                glVertex3f(l div 2,0,j+1);
              glEnd;

              intens2 := (j+ (b div 2));
              if intens2 = 0 then intens2 := 1/(l+b);
              intens1 := 2*intens2 / (l+b);
              intens2 := intens1 + ( 1/(l+b) );

              glBegin(GL_LINES);
                glColor3f(color[0]*intens1, color[1]*intens1, color[2]*intens1);
                glVertex3f(-l div 2,0,j);

                glColor3f(color[0]*intens2, color[1]*intens2, color[2]*intens2);
                glVertex3f(-l div 2,0,j+1);
              glEnd;
         end;

     if active then drawArrows;

     glLineWidth(1);

     glEnable(GL_DEPTH_TEST);
end;

procedure TGrid.drawArrows;
begin
     glBegin(GL_LINES);
       glColor3f(0.5,0.0,0.0); glVertex3f(-1,0,0);
       glColor3f(1.0,0.0,0.0); glVertex3f(1,0,0);
       glVertex3f(0.8,0,0.1);  glVertex3f(1,0,0);
       glVertex3f(0.8,0,-0.1); glVertex3f(1,0,0);

       glColor3f(0.0,0.0,0.5); glVertex3f(0,0,-1);
       glColor3f(0.0,0.0,1.0); glVertex3f(0,0,1);
       glVertex3f(0.1,0,0.8);  glVertex3f(0,0,1);
       glVertex3f(-0.1,0,0.8); glVertex3f(0,0,1);
     glEnd;
end;

procedure drawScene(Step : integer);
begin
     case Step of
          0: begin
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          1: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;

                  glTranslatef(3.5,0,-3);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          2: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;
                  glTranslatef(3.5,0,-3);
                  aGrid.drawGrid;

                  glRotatef(45, 0, 1, 0);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          3: begin
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          4: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;

                  glRotatef(45, 0, 1, 0);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          5: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;
                  glRotatef(45, 0, 1, 0);
                  aGrid.drawGrid;

                  glTranslatef(3.5,0,-3);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          6: begin
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          7: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;

                  glScalef(0.8,1,0.6);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          8: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;
                  glScalef(0.8,1,0.6);
                  aGrid.drawGrid;

                  glRotatef(45,0,1,0);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
          9: begin
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
         10: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;

                  glScalef(0.8,1,0.6);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
         11: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;
                  glScalef(0.8,1,0.6);
                  aGrid.drawGrid;

                  glTranslatef(3.5,0,-3);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
         12: begin
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
         13: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;

                  glRotatef(45, 0, 1, 0);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
         14: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;
                  glRotatef(45, 0, 1, 0);
                  aGrid.drawGrid;

                  glRotatef(45,1,0,0);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
         15: begin
                  aGrid.Active := false;
                  aGrid.drawGrid;
                  glRotatef(45, 0, 1, 0);
                  aGrid.drawGrid;
                  glRotatef(45, 1, 0, 0);
                  aGrid.drawGrid;

                  glTranslatef(3,0,0);
                  aGrid.Active := true;
                  aGrid.drawGrid;
             end;
     end;
end;





end.
